/*
 *  InputHandler.h
 *  Hubschrauber
 *
 *  Created by Torsten on 17.09.06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#ifdef __APPLE_CC__
#include <SDL/SDL.h>
#else
#include "SDL.h"
#endif

#include <string>

namespace Game
{

class World;

class InputHandler
{
	public:
	
	enum AxisPart
	{
		AxisPartPositive,
		AxisPartNegative
	};
	enum AxisMode
	{
		AxisModeFull,
		AxisModeHalf
	};
	enum Type
	{
		MouseMove = 1,
		MouseClick,
		Keyboard,
		JoystickAxis,
		JoystickBall, // WTF is that?
		JoystickHat,
		JoystickButton
	};
	
	typedef void (*InputHandlerFunction) (InputHandler *handler, float value);
	
	InputHandler(World *aWorld, std::string preferenceKey, AxisMode axisMode, InputHandlerFunction aFunc, Type defaultType, unsigned defaultElement, AxisPart defaultAxisPart);
	
	std::wstring currentlyBoundName();
	
	bool tryBindToEvent(SDL_Event *event);
	void HandleEvent(SDL_Event *event);
	bool currentlyBound() { return isBound; };
	void unbind();
	
	void SetNegatesValue(bool value) { negate = value; }
	bool GetNegatesValue() { return negate; }
	
	float GetValue() { return value; } // ALWAYS from 0 to 1

	World *GetWorld() { return world; }

	static void SetMouseFactor(float factor);
	static float GetMouseFactor();



	private:
	
	std::string PreferenceValue();
	static void ReloadMouseFactor();
	
	World *world;
	
	InputHandlerFunction func;
	
	bool negate;
	bool isBound;
	float value;
	
	std::wstring deviceElementName;
	std::string preferenceKey;
	
	AxisMode mode;
	Type type;
	unsigned element;
	AxisPart partOfAxis;
};

}